using System;
using System.Runtime.InteropServices; 

namespace Synoxo.USBHidDevice
{
	public partial class DeviceManagement
	{
		///<summary >
		// API declarations relating to device management (SetupDixxx and 
		// RegisterDeviceNotification functions).   
		/// </summary>

		// from dbt.h
		public const Int32 DBT_DEVICEARRIVAL = 0X8000;
		public const Int32 DBT_DEVICEREMOVECOMPLETE = 0X8004;
		public const Int32 DBT_DEVTYP_DEVICEINTERFACE = 5;
		public const Int32 DBT_DEVTYP_HANDLE = 6;
		public const Int32 DEVICE_NOTIFY_ALL_INTERFACE_CLASSES = 4;
		public const Int32 DEVICE_NOTIFY_SERVICE_HANDLE = 1;
		public const Int32 DEVICE_NOTIFY_WINDOW_HANDLE = 0;
		public const Int32 WM_DEVICECHANGE = 0X219;

		// from setupapi.h
		public const Int32 DIGCF_PRESENT = 2;
		public const Int32 DIGCF_DEVICEINTERFACE = 0X10;

		// Two declarations for the DEV_BROADCAST_DEVICEINTERFACE structure.

		// Use this one in the call to RegisterDeviceNotification() and
		// in checking dbch_devicetype in a DEV_BROADCAST_HDR structure:

		[StructLayout(LayoutKind.Sequential)]
		public class DEV_BROADCAST_DEVICEINTERFACE
		{
			public Int32 dbcc_size;
			public Int32 dbcc_devicetype;
			public Int32 dbcc_reserved;
			public Guid dbcc_classguid;
			public Int16 dbcc_name;
		}

		// Use this to read the dbcc_name String and classguid:
		[StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
		public class DEV_BROADCAST_DEVICEINTERFACE_1
		{
			public Int32 dbcc_size;
			public Int32 dbcc_devicetype;
			public Int32 dbcc_reserved;
			[MarshalAs(UnmanagedType.ByValArray, ArraySubType = UnmanagedType.U1, SizeConst = 16)]
			public Byte[] dbcc_classguid;
			[MarshalAs(UnmanagedType.ByValArray, SizeConst = 255)]
			public Char[] dbcc_name;
		}

		[StructLayout(LayoutKind.Sequential)]
		public class DEV_BROADCAST_HDR
		{
			public Int32 dbch_size;
			public Int32 dbch_devicetype;
			public Int32 dbch_reserved;
		}

		public struct SP_DEVICE_INTERFACE_DATA
		{
			public Int32 cbSize;
			public System.Guid InterfaceClassGuid;
			public Int32 Flags;
			public IntPtr Reserved;
		}

		public struct SP_DEVICE_INTERFACE_DETAIL_DATA
		{
			public Int32 cbSize;
			public String DevicePath;
		}

		public struct SP_DEVINFO_DATA
		{
			public Int32 cbSize;
			public System.Guid ClassGuid;
			public Int32 DevInst;
			public Int32 Reserved;
		}

        /// <summary>
        ///  ӿӻ߰γʱ֪ͨϢ
        /// Request to receive notification messages when a device in an interface class is attached or removed.
        /// </summary>
        /// <param name="hRecipient">豸¼Ĵھ
        /// Handle to the window that will receive device events</param>
        /// <param name="NotificationFilter">¼豸ṹָ
        /// Pointer to a DEV_BROADCAST_DEVICEINTERFACE to specify the type of device to send notifications for.</param>
        /// <param name="Flags">DEVICE_NOTIFY_WINDOW_HANDLE indicates the handle is a window handle.</param>
        /// <returns>Device notification handle or NULL on failure.</returns>
		[DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
		public static extern IntPtr RegisterDeviceNotification(IntPtr hRecipient, IntPtr NotificationFilter, Int32 Flags);

        /// <summary>
        /// ֹͣϵͳϢ
        /// </summary>
        /// <param name="Handle">Handle returned previously by RegisterDeviceNotification.</param>
        /// <returns>True on success</returns>
		[DllImport("user32.dll", SetLastError = true)]
		public static extern Boolean UnregisterDeviceNotification(IntPtr Handle);

		[DllImport("setupapi.dll", SetLastError = true)]
		public static extern Int32 SetupDiCreateDeviceInfoList(ref System.Guid ClassGuid, Int32 hwndParent);

        /// <summary>
        /// ͷΪDeviceInfoSetصSetupDiGetClassDevsڴ
        /// </summary>
        /// <param name="DeviceInfoSet">SetupDiGetClassDevs ص DeviceInfoSet</param>
        /// <returns>True on success</returns>
		[DllImport("setupapi.dll", SetLastError = true)]
		public static extern Int32 SetupDiDestroyDeviceInfoList(IntPtr DeviceInfoSet);

        ///  <summary>
        ///  Retrieves a handle to a SP_DEVICE_INTERFACE_DATA structure for a device.
        ///  On return, MyDeviceInterfaceData contains the handle to a
        ///  SP_DEVICE_INTERFACE_DATA structure for a detected device.
        ///  </summary>
        ///  <param name="DeviceInfoSet">DeviceInfoSet returned by SetupDiGetClassDevs</param>
        ///  <param name="DeviceInfoData">Optional SP_DEVINFO_DATA structure that defines a device instance that is a member of a device information set</param> 
        ///  <param name="InterfaceClassGuid">Device interface GUID</param>
        ///  <param name="MemberIndex">Index to specify a device in a device information set</param>
        ///  <param name="DeviceInterfaceData">ָ豸SP_DEVICE_INTERFACE_DATAṹָ</param>
        ///  <returns>True on success.</returns>
		[DllImport("setupapi.dll", SetLastError = true)]
		public static extern Boolean SetupDiEnumDeviceInterfaces(IntPtr DeviceInfoSet, IntPtr DeviceInfoData, ref System.Guid InterfaceClassGuid, Int32 MemberIndex, ref SP_DEVICE_INTERFACE_DATA DeviceInterfaceData);

        ///  <summary>
        ///   Ϊָ͵豸µõ豸Ϣ
        ///   Retrieves a device information set for a specified group of devices.
        ///  </summary>
        ///  <param name="ClassGuid">SetupDiEnumDeviceInterfaces uses the device information set Interface class GUID.</param>
        ///  <param name="Enumerator">Null to retrieve information for all device instances.</param>
        ///  <param name="hwndParent">Optional handle to a top-level window (unused here)</param>
        ///  <param name="Flags">Flags to limit the returned information to currently present devices 
        ///  and devices that expose interfaces in the class specified by the GUID
        ///  </param>
        ///  <returns>Handle to a device information set for the devices.</returns>
		[DllImport("setupapi.dll", SetLastError = true, CharSet = CharSet.Auto)]
		public static extern IntPtr SetupDiGetClassDevs(ref System.Guid ClassGuid, IntPtr Enumerator, IntPtr hwndParent, Int32 Flags);

        /// <summary>
        /// һSP_DEVICE_INTERFACE_DETAIL_DATAṹԴ洢豸Ϣ
        /// ҪϢõô˺,һεõṹС,ڶεõָݵָ
        /// </summary>
        /// <param name="DeviceInfoSet">SetupDiGetClassDevsص豸Ϣ</param>
        /// <param name="DeviceInterfaceData">SetupDiEnumDeviceInterfacesصSP_DEVICE_INTERFACE_DATAݽṹ</param>
        /// <param name="DeviceInterfaceDetailData">صĽϢָSP_DEVICE_INTERFACE_DETAIL_DATAָ</param>
        /// <param name="DeviceInterfaceDetailDataSize">SP_DEVICE_INTERFACE_DETAIL_DATAݽṹĴС</param>
        /// <param name="RequiredSize">ָҪշصSP_DEVICE_INTERFACE_DETAIL_DATA ݽṹСָ</param>
        /// <param name="DeviceInfoData">ص豸ϢSP_DEVINFO_DATAṹָ</param>
        /// <returns>ɹtrue</returns>
		[DllImport("setupapi.dll", SetLastError = true, CharSet = CharSet.Auto)]
		public static extern Boolean SetupDiGetDeviceInterfaceDetail(IntPtr DeviceInfoSet, ref SP_DEVICE_INTERFACE_DATA DeviceInterfaceData, IntPtr DeviceInterfaceDetailData, Int32 DeviceInterfaceDetailDataSize, ref Int32 RequiredSize, IntPtr DeviceInfoData);
	}
}
