using Microsoft.Win32.SafeHandles; 
using System;
using System.Runtime.InteropServices;

namespace Synoxo.USBHidDevice
{
    [StructLayout(LayoutKind.Sequential)]
    public struct HIDD_ATTRIBUTES
    {
        public Int32 Size;
        public UInt16 VendorID;
        public UInt16 ProductID;
        public UInt16 VersionNumber;
    }

    public struct HIDP_CAPS
    {
        public Int16 Usage;
        public Int16 UsagePage;
        public Int16 InputReportByteLength;
        public Int16 OutputReportByteLength;
        public Int16 FeatureReportByteLength;
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 17)]
        public Int16[] Reserved;
        public Int16 NumberLinkCollectionNodes;
        public Int16 NumberInputButtonCaps;
        public Int16 NumberInputValueCaps;
        public Int16 NumberInputDataIndices;
        public Int16 NumberOutputButtonCaps;
        public Int16 NumberOutputValueCaps;
        public Int16 NumberOutputDataIndices;
        public Int16 NumberFeatureButtonCaps;
        public Int16 NumberFeatureValueCaps;
        public Int16 NumberFeatureDataIndices;
    }

    //  If IsRange is false, UsageMin is the Usage and UsageMax is unused.
    //  If IsStringRange is false, StringMin is the String index and StringMax is unused.
    //  If IsDesignatorRange is false, DesignatorMin is the designator index and DesignatorMax is unused.
    public struct HidP_Value_Caps
    {
        public Int16 UsagePage;
        public Byte ReportID;
        public Int32 IsAlias;
        public Int16 BitField;
        public Int16 LinkCollection;
        public Int16 LinkUsage;
        public Int16 LinkUsagePage;
        public Int32 IsRange;
        public Int32 IsStringRange;
        public Int32 IsDesignatorRange;
        public Int32 IsAbsolute;
        public Int32 HasNull;
        public Byte Reserved;
        public Int16 BitSize;
        public Int16 ReportCount;
        public Int16 Reserved2;
        public Int16 Reserved3;
        public Int16 Reserved4;
        public Int16 Reserved5;
        public Int16 Reserved6;
        public Int32 LogicalMin;
        public Int32 LogicalMax;
        public Int32 PhysicalMin;
        public Int32 PhysicalMax;
        public Int16 UsageMin;
        public Int16 UsageMax;
        public Int16 StringMin;
        public Int16 StringMax;
        public Int16 DesignatorMin;
        public Int16 DesignatorMax;
        public Int16 DataIndexMin;
        public Int16 DataIndexMax;
    }

    internal partial class Hid
    {
        //  API declarations for HID communications.

        //  from hidpi.h
        //  Typedef enum defines a set of integer constants for HidP_Report_Type

        public const Int16 HidP_Input = 0;
        public const Int16 HidP_Output = 1;
        public const Int16 HidP_Feature = 2;

        /// <summary>
        /// Ƴȴڻеı
        /// </summary>
        /// <param name="HidDeviceObject">ָ豸ľ</param>
        /// <returns>ɹTrue , ʧܣFalse</returns>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern Boolean HidD_FlushQueue(SafeFileHandle HidDeviceObject);

        /// <summary>
        /// ͷHidD_GetPreparsedDataĻ
        /// </summary>
        /// <param name="PreparsedData">ָHidD_GetPreparsedDataصPreparsedDataݽṹָ</param>
        /// <returns>ɹTrue , ʧܣFalse</returns>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern Boolean HidD_FreePreparsedData(IntPtr PreparsedData);

        /// <summary>
        /// HID豸ȡGUIDRetrieves the interface class GUID for the HID class.
        /// </summary>
        /// <param name="HidGuid">ϵͳ豸GUIDַA System.Guid object for storing the GUID</param>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern void HidD_GetHidGuid(ref System.Guid HidGuid);

        /// <summary>
        /// ȡ豸
        /// </summary>
        /// <param name="HidDeviceObject">豸</param>
        /// <param name="Attributes">HIDD_ATTRIBUTESṹ</param>
        /// <returns>ɹTrue , ʧܣFalse</returns>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern Boolean HidD_GetAttributes(SafeFileHandle HidDeviceObject, ref HIDD_ATTRIBUTES Attributes);

        /// <summary>
        /// Ŀ: ȡ豸ܲ
        /// ڱ׼豸磺ֱҳ豸
        /// ֪Ŀͻ豸Ҫ˺
        /// </summary>
        /// <param name="PreparsedData">ָHidD_GetPreparsedDataصĽṹָ</param>
        /// <param name="Capabilities">ָHIDP_CAPS structureĽṹָ</param>
        /// <returns>ɹTrue , ʧܣFalse</returns>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern Int32 HidP_GetCaps(IntPtr PreparsedData, ref HIDP_CAPS Capabilities);

        /// <summary>
        /// ŷԱ豸Attempts to send a Feature report to the device
        /// </summary>
        /// <param name="HidDeviceObject">HID豸</param>
        /// <param name="lpReportBuffer">ָ򱨸IDͱĻָ</param>
        /// <param name="ReportBufferLength">С</param>
        /// <returns>ɹTrue , ʧܣFalse</returns>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern Boolean HidD_SetFeature(SafeFileHandle HidDeviceObject, Byte[] lpReportBuffer, Int32 ReportBufferLength);

        /// <summary>
        /// Ŷȡ豸ı
        /// </summary>
        /// <param name="HidDeviceObject">HID豸</param>
        /// <param name="lpReportBuffer">IDݵĻָ</param>
        /// <param name="ReportBufferLength"></param>
        /// <returns>ɹTrue , ʧܣFalse</returns>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern Boolean HidD_GetFeature(SafeFileHandle HidDeviceObject, Byte[] lpReportBuffer, Int32 ReportBufferLength);

        /// <summary>
        /// ÿƴ䷢浽豸
        /// ҪWin XPԺİ汾֧
        /// </summary>
        /// <param name="HidDeviceObject">HID豸</param>
        /// <param name="lpReportBuffer">ָ򱨸IDͱָ</param>
        /// <param name="ReportBufferLength">泤</param>
        /// <returns>ɹTrue , ʧܣFalse</returns>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern Boolean HidD_SetOutputReport(SafeFileHandle HidDeviceObject, Byte[] lpReportBuffer, Int32 ReportBufferLength);

        /// <summary>
        /// ÿƴ䣨control transferŴHID豸ȡı棬֧Windows XPԺİ汾
        /// </summary>
        /// <param name="HidDeviceObject">HID豸</param>
        /// <param name="lpReportBuffer">IDݵĻָ</param>
        /// <param name="ReportBufferLength"></param>
        /// <returns>ɹTrue , ʧܣFalse</returns>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern Boolean HidD_GetInputReport(SafeFileHandle HidDeviceObject, Byte[] lpReportBuffer, Int32 ReportBufferLength);

        /// <summary>
        /// Sets the number of Input reports the host can store
        /// ܴ洢뱨
        /// </summary>
        /// <param name="HidDeviceObject">HID豸</param>
        /// <param name="NumberBuffers">An integer to hold the number of buffers</param>
        /// <returns>ɹTrue , ʧܣFalse</returns>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern Boolean HidD_SetNumInputBuffers(SafeFileHandle HidDeviceObject, Int32 NumberBuffers);

        /// <summary>
        /// ȡHostܴ洢뱨ֵ֧Windows 98 Goldµı浽򶪵ı(ldest report)
        /// </summary>
        /// <param name="HidDeviceObject">HID豸a handle to a device</param>
        /// <param name="NumberBuffers">ֵֻinteger to hold the number of buffers</param>
        /// <returns>ɹTrue , ʧܣFalse</returns>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern Boolean HidD_GetNumInputBuffers(SafeFileHandle HidDeviceObject, ref Int32 NumberBuffers);

        /// <summary>
        /// API: HidD_GetPreparsedData
        /// Ŀ: ȡָϢṹָ.
        /// HidP_GetCaps  API Ҫһָ򻺳ָ.
        /// </summary>
        /// <param name="HidDeviceObject">һCreateFileصļ</param>
        /// <param name="PreparsedData">һָBuffָ</param>
        /// <returns>ɹTrue , ʧܣFalse</returns>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern Boolean HidD_GetPreparsedData(SafeFileHandle HidDeviceObject, ref IntPtr PreparsedData);

        /// <summary>
        /// »ȡһ HidP_ValueCapsṹУÿṹһֵ˳δ
        /// retrieves a buffer containing an array of HidP_ValueCaps structures.
        ///  Each structure defines the capabilities of one value.
        ///  This application doesn't use this data.
        /// </summary>
        /// <param name="ReportType">hidpi.hеıöͣA report type enumerator from hidpi.h</param>
        /// <param name="ValueCaps">лָ룻A pointer to a buffer for the returned array</param>
        /// <param name="ValueCapsLength">豸HidP_Capsṹ洢The NumberInputValueCaps member of the device's HidP_Caps structure</param>
        /// <param name="PreparsedData">HidD_GetPreparsedDataصPreparsedDataݽṹָ룬A pointer to the PreparsedData structure returned by HidD_GetPreparsedData</param>
        /// <returns>ɹTrue , ʧܣFalse</returns>
        [DllImport("hid.dll", SetLastError = true)]
        public static extern Int32 HidP_GetValueCaps(Int32 ReportType, Byte[] ValueCaps, ref Int32 ValueCapsLength, IntPtr PreparsedData);
    }
}
